/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

//*****************************************************************************
// Development main.c for MSP430FR26xx_25xx
//
// This starter application initializes the CapTIvate touch library
// for the touch panel specified by CAPT_UserConfig.c/.h via a call to
// CAPT_appStart(), which initializes and calibrates all sensors in the
// application, and starts the CapTIvate interval timer.
//
// Then, the capacitive touch interface is driven by calling the CapTIvate
// application handler, CAPT_appHandler().  The application handler manages
// whether the user interface (UI) is running in full active scan mode, or
// in a low-power wake-on-proximity mode.
//
// The CapTIvate application handler will return true if proximity was
// detected on any of the sensors in the application. After the application
// handler, the number of touches detected on the sensor will be counted.
// If a single touch is detected, only LED1 will light up. If multiple
// touches are detected, both LED1 and LED2 light up.
//
// \version 1.01.01.00
// Released on Thu, Oct 15, 2015 12:27:24 AM
//
//*****************************************************************************

//*****************************************************************************
//! \addtogroup Basic_Demo_Main
//! @{
//*****************************************************************************

#include <msp430.h>                      // Generic MSP430 Device Include
#include "driverlib.h"                   // MSPWare Driver Library
#include "captivate.h"                   // CapTIvate Touch Software Library
#include "CAPT_App.h"                    // CapTIvate Application Code
#include "CAPT_BSP.h"                    // CapTIvate EVM Board Support Package

//*****************************************************************************
//
//! \brief Size of the general purpose data packet
//
//*****************************************************************************
#define ROW_PACKET_SIZE 16

//*****************************************************************************
//
//! \brief Index of keypad in sensor list
//
//*****************************************************************************
#define KEYPAD_SENSOR 0

//*****************************************************************************
//
//! \brief General purpose data packet used in testing/debug
//
//*****************************************************************************
uint16_t row1Data[ROW_PACKET_SIZE];

void main(void)
{
	//
	// Initialize the MCU
	// BSP_configureMCU() sets up the device IO and clocking
	// The global interrupt enable is set to allow peripherals
	// to wake the MCU.
	//
	WDT_A_hold(WDT_A_BASE);
	BSP_configureMCU();
	__bis_SR_register(GIE);

	//
	// Start the CapTIvate application
	//
	CAPT_appStart();

	//
	// Background Loop
	//
	while(1)
	{
		//
		// Run the captivate application handler.
		//
		CAPT_appHandler();

		//
		// Background application that keeps track
		// of the number of buttons on the sensor
		// that are currently in a touch state.
		//
		if(g_uiApp.pSensorList[0]->bSensorTouch)
		{
			uint8_t cycle, element, touchCount = 0;

			for(cycle = 0; cycle < g_uiApp.pSensorList[KEYPAD_SENSOR]->ui8NrOfCycles; cycle++)
			{
				for(element = 0; element < g_uiApp.pSensorList[KEYPAD_SENSOR]->pCycle[cycle]->ui8NrOfElements; element++)
				{
					if(g_uiApp.pSensorList[KEYPAD_SENSOR]->pCycle[cycle]->pElements[element]->bTouch)
					{
						touchCount++;
					}
				}
			}

			LED1_ON;
			if(touchCount > 1)
			{
				LED2_ON;
			}
			else
			{
				LED2_OFF;
			}
		}
		else
		{
			LED1_OFF;
			LED2_OFF;
		}


#if ((CAPT_INTERFACE == __CAPT_UART_INTERFACE__)||\
		(CAPT_INTERFACE == __CAPT_BULKI2C_INTERFACE__))

		// Build and send updated raw counts for row 1 buttons
		// over HID in general purpose data packet
		row1Data[0] = g_uiApp.pSensorList[0]->pCycle[0]->pElements[0]->pRawCount[0];
		row1Data[1] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[0]->pElements[0]->bTouch;

		row1Data[2] = g_uiApp.pSensorList[0]->pCycle[1]->pElements[0]->pRawCount[0];
		row1Data[3] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[1]->pElements[0]->bTouch;

		row1Data[4] = g_uiApp.pSensorList[0]->pCycle[0]->pElements[1]->pRawCount[0];
		row1Data[5] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[0]->pElements[1]->bTouch;

		row1Data[6] = g_uiApp.pSensorList[0]->pCycle[1]->pElements[1]->pRawCount[0];
		row1Data[7] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[1]->pElements[1]->bTouch;

		row1Data[8] = g_uiApp.pSensorList[0]->pCycle[0]->pElements[2]->pRawCount[0];
		row1Data[9] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[0]->pElements[2]->bTouch;

		row1Data[10] = g_uiApp.pSensorList[0]->pCycle[1]->pElements[2]->pRawCount[0];
		row1Data[11] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[1]->pElements[2]->bTouch;

		row1Data[12] = g_uiApp.pSensorList[0]->pCycle[0]->pElements[3]->pRawCount[0];
		row1Data[13] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[0]->pElements[3]->bTouch;

		row1Data[14] = g_uiApp.pSensorList[0]->pCycle[1]->pElements[3]->pRawCount[0];
		row1Data[15] = (uint16_t)g_uiApp.pSensorList[0]->pCycle[1]->pElements[3]->bTouch;

		CAPT_writeGeneralPurposeData(row1Data, ROW_PACKET_SIZE);
#endif

		//
		// End of background loop iteration
		// Go to sleep if there is nothing left to do
		//
		CAPT_appSleep();
		
	} // End background loop
} // End main()

//*****************************************************************************
//! Close the doxygen group
//! @}
//*****************************************************************************
