/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// Development main.c for MSP430FR26xx_25xx
//
// This starter application initializes the CapTIvate touch library
// for the touch panel specified by CAPT_UserConfig.c/.h via a call to
// CAPT_appStart(), which initializes and calibrates all sensors in the
// application, and starts the CapTIvate interval timer.
//
// Then, the capacitive touch interface is driven by calling the CapTIvate
// application handler, CAPT_appHandler().  The application handler manages
// whether the user interface (UI) is running in full active scan mode, or
// in a low-power wake-on-proximity mode.
//
// The CapTIvate application handler will return true if proximity was
// detected on any of the sensors in the application. If RESPONSE_TESTING is
// not defined, the number of buttons in a touch state is counted following the 
// application handler. If a single touch is detected, LED1 will turn on. If 
// more than one touch is detected, LED1 and LED2 will turn on.
//
// \version 1.01.01.00
// Released on Thu, Oct 15, 2015 12:27:24 AM
//
//*****************************************************************************

//*****************************************************************************
//! \addtogroup Response_Optimization_Main
//! @{
//*****************************************************************************

#include <msp430.h>                      // Generic MSP430 Device Include
#include "captivate.h"                   // CapTIvate Touch Software Library

//*****************************************************************************
// Definitions
//*****************************************************************************

//*****************************************************************************
//
//! \brief The bit mask for LED1 on port P1.
//
//*****************************************************************************
#define LED1 0x80

//*****************************************************************************
//
//! \brief The bit mask for LED2 on port P1.
//
//*****************************************************************************
#define LED2 0x40

//*****************************************************************************
//
//! \brief The ratio between the DCOCLK and the FLLREF.
//
//*****************************************************************************
#define FLL_RATIO 0x01E7

//*****************************************************************************
//
//! \brief Allows keypad handler to run response testing code when uncommented
//
//*****************************************************************************
//#define RESPONSE_TESTING

//*****************************************************************************
// Function Prototypes
//*****************************************************************************

//*****************************************************************************
//
//! Event handler for the keypad sensor.
//! This handler updates LED1 and LED2 based on
//! the status of the keypad sensor.
//
//! \param Pointer to the keypad sensor
//
//*****************************************************************************
static void keypadHandler(tSensor* pSensor);

void main(void)
{
	//
	// Stop the watchdog timer
	//
	WDTCTL = WDTPW | WDTHOLD;

	//
	// Set the port muxing:
	//  - P1.1: IRQ (OPEN DRAIN)
	//  - P1.2: UCB0 I2C SDA
	//  - P1.3: UCB0 I2C SCL
	//  - P1.4: UCA0 UART TXD
	//  - P1.5: UCA0 UART RXD
	//  - P1.6: LED2
	//  - P1.7: LED1
	//  - P2.0: XOUT
	//  - P2.1: XIN
	//
	PAOUT  = 0x0000;
	PADIR  = 0xFCC1;
	PASEL0 = 0x033C;
	P3OUT  = 0x00;
	P3DIR  = 0x07;
	P3SEL0 = 0x00;
	PM5CTL0 &= ~LOCKLPM5;

	//
	// Set up FRAM wait state to allow for a 16 MHz MCLK.
	// Note that CapTIvate ROM functions execute without
	// any wait states at 16 MHz.
	//
	FRCTL0 = FRCTLPW | NWAITS_1;

	//
	// Set up the system clocks.
	// MCLK = DCO = 16 MHz
	// ACLK = XT1 = 32 kHz
	// SMCLK = OFF
	//
	do
	{
		CSCTL7 &= ~(XT1OFFG | DCOFFG);
		SFRIFG1 &= ~OFIFG;
	} while (SFRIFG1 & OFIFG);
	CSCTL3 = SELREF__XT1CLK;
	__bis_SR_register(SCG0);
	CSCTL0 = 0;
	CSCTL1 &= ~(DCORSEL_7);
	CSCTL1 |= DCORSEL_5;
	CSCTL2 = FLLD_0 + FLL_RATIO;
	__delay_cycles(3);
	__bic_SR_register(SCG0);
	while(CSCTL7 & (FLLUNLOCK0 | FLLUNLOCK1));
	CSCTL4 = SELMS__DCOCLKDIV | SELA__XT1CLK;
	CSCTL5 = SMCLKOFF | VLOAUTOOFF;

	//
	// Set up the CapTIvate peripheral
	// and timer.  The timer will periodically
	// set the g_bConvTimerFlag.
	//
	CAPT_initUI(&g_uiApp);
	CAPT_calibrateUI(&g_uiApp);
	MAP_CAPT_registerCallback(&keypad64Button, &keypadHandler);
	MAP_CAPT_selectTimerSource(CAPT_TIMER_SRC_ACLK);
	MAP_CAPT_writeTimerCompRegister(CAPT_MS_TO_CYCLES(g_uiApp.ui16ActiveModeScanPeriod));
	MAP_CAPT_startTimer();
	MAP_CAPT_enableISR(CAPT_TIMER_INTERRUPT);
	__bis_SR_register(GIE);

	//
	// Background Loop
	//
	while(1)
	{
		__bic_SR_register(GIE);

		if (g_bConvTimerFlag == true)
		{
			//
			// If it is time to update the button,
			// update it here with the generic library call.
			//
			g_bConvTimerFlag = false;
			CAPT_updateUI(&g_uiApp);

			// CapTIvate Technology to power off
			MAP_CAPT_clearCAPLPMCFG();
		}

		//
		// Go to sleep when finished if flag hasn't been set.
		// Otherwise, run update again.
		// The CapTIvate timer will wake the application from sleep.
		//
		if(g_bConvTimerFlag == true)
		{
			__bis_SR_register(GIE);
		}
		else
		{
			__bis_SR_register(g_uiApp.ui8AppLPM | GIE);
		}
		
	} // End background loop
} // End main()

void keypadHandler(tSensor* pSensor)
{
#ifndef RESPONSE_TESTING
	if(pSensor->bSensorTouch)
	{
		uint8_t cycle, element, touchCount = 0;

		for(cycle = 0; cycle < pSensor->ui8NrOfCycles; cycle++)
		{
			for(element = 0; element < pSensor->pCycle[cycle]->ui8NrOfElements; element++)
			{
				if(pSensor->pCycle[cycle]->pElements[element]->bTouch)
				{
					touchCount++;
				}
			}
		}

		P1OUT |= LED1;
		if(touchCount > 1)
		{
			P1OUT |= LED2;
		}
		else
		{
			P1OUT &= ~LED2;
		}
	}
	else
	{
		P1OUT &= ~LED1;
		P1OUT &= ~LED2;
	}
#else
	// Set P1.7/P1.6 (LED1 and LED2) according to Element 00 status
	if(pSensor->pCycle[0]->pElements[0]->bDetect && !pSensor->pCycle[0]->pElements[0]->bTouch)
	{
		P1OUT |= LED1;
		P1OUT &= ~LED2;
	}
	else if(!pSensor->pCycle[0]->pElements[0]->bDetect && pSensor->pCycle[0]->pElements[0]->bTouch)
	{
		P1OUT &= ~LED1;
		P1OUT |= LED2;
	}
	else if (pSensor->pCycle[0]->pElements[0]->bDetect && pSensor->pCycle[0]->pElements[0]->bTouch)
	{
		P1OUT |= LED1;
		P1OUT |= LED2;
	}
	else
	{
		P1OUT &= ~LED1;
		P1OUT &= ~LED2;
	}
#endif
}

//*****************************************************************************
//! Close the doxygen group
//! @}
//*****************************************************************************
